@@ -3,7 +3,7 @@
 -- SPDX-FileCopyrightText: Copyright (c) 2025 Objectionary.com
 -- SPDX-License-Identifier: MIT
 
-module Functions where
+module Functions (buildTerm) where
 
 import Ast
 import Builder
@@ -26,26 +26,44 @@ import Term
 import Text.Printf (printf)
 import qualified Yaml as Y
 
+buildTerm :: String -> [Y.ExtraArgument] -> Subst -> Program -> IO Term
+buildTerm "contextualize" = _contextualize
+buildTerm "scope" = _scope
+buildTerm "random-tau" = _randomTau
+buildTerm "dataize" = _dataize
+buildTerm "concat" = _concat
+buildTerm "sed" = _sed
+buildTerm "random-string" = _randomString
+buildTerm "size" = _size
+buildTerm "tau" = _tau
+buildTerm "string" = _string
+buildTerm "number" = _number
+buildTerm func  = _unsupported func
+
 argToStrBytes :: Y.ExtraArgument -> Subst -> Program -> IO String
 argToStrBytes (Y.ArgBytes bytes) subst _ = do
   bts <- buildBytesThrows bytes subst
   pure (btsToUnescapedStr bts)
 argToStrBytes (Y.ArgExpression expr) subst prog = do
-  (TeBytes bts) <- buildTermFromFunction "dataize" [Y.ArgExpression expr] subst prog
+  (TeBytes bts) <- _dataize [Y.ArgExpression expr] subst prog
   pure (btsToUnescapedStr bts)
 argToStrBytes arg _ _ = throwIO (userError (printf "Can't extract bytes from given argument: %s" (prettyExtraArg arg)))
 
-buildTermFromFunction :: String -> [Y.ExtraArgument] -> Subst -> Program -> IO Term
-buildTermFromFunction "contextualize" [Y.ArgExpression expr, Y.ArgExpression context] subst prog = do
+_contextualize :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_contextualize [Y.ArgExpression expr, Y.ArgExpression context] subst prog = do
   (expr', _) <- buildExpressionThrows expr subst
   (context', _) <- buildExpressionThrows context subst
   pure (TeExpression (contextualize expr' context' prog))
-buildTermFromFunction "contextualize" _ _ _ = throwIO (userError "Function contextualize() requires exactly 2 arguments as expression")
-buildTermFromFunction "scope" [Y.ArgExpression expr] subst _ = do
-  (expr', scope) <- buildExpressionThrows expr subst
+_contextualize _ _ _ = throwIO (userError "Function contextualize() requires exactly 2 arguments as expression")
+
+_scope :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_scope [Y.ArgExpression expr] subst _ = do
+  (_, scope) <- buildExpressionThrows expr subst
   pure (TeExpression scope)
-buildTermFromFunction "scope" _ _ _ = throwIO (userError "Function scope() requires exactly 1 argument as expression")
-buildTermFromFunction "random-tau" args subst _ = do
+_scope _ _ _ = throwIO (userError "Function scope() requires exactly 1 argument as expression")
+
+_randomTau :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_randomTau args subst _ = do
   attrs <- argsToAttrs args
   tau <- randomTau attrs
   pure (TeAttribute (AtLabel tau))
@@ -76,19 +94,25 @@ buildTermFromFunction "random-tau" args subst _ = do
     randomTau attrs = do
       tau <- randomString "a🌵%d"
       if tau `elem` attrs then randomTau attrs else pure tau
-buildTermFromFunction "dataize" [Y.ArgBytes bytes] subst _ = do
+
+_dataize :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_dataize [Y.ArgBytes bytes] subst _ = do
   bts <- buildBytesThrows bytes subst
   pure (TeBytes bts)
-buildTermFromFunction "dataize" [Y.ArgExpression expr] subst _ = do
+_dataize [Y.ArgExpression expr] subst _ = do
   (expr', _) <- buildExpressionThrows expr subst
   case expr' of
     DataObject _ bytes -> pure (TeBytes bytes)
     _ -> throwIO (userError "Only data objects and bytes are supported by 'dataize' function now")
-buildTermFromFunction "dataize" _ _ _ = throwIO (userError "Function dataize() requires exactly 1 argument as expression")
-buildTermFromFunction "concat" args subst prog = do
+_dataize _ _ _ = throwIO (userError "Function dataize() requires exactly 1 argument as expression or bytes")
+
+_concat :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_concat args subst prog = do
   args' <- traverse (\arg -> argToStrBytes arg subst prog) args
   pure (TeExpression (DataString (strToBts (concat args'))))
-buildTermFromFunction "sed" args subst prog = do
+
+_sed :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_sed args subst prog = do
   when (length args < 2) (throwIO (userError "Function sed() requires at least two arguments"))
   args' <-
     traverse
@@ -131,21 +155,29 @@ buildTermFromFunction "sed" args subst prog = do
             Nothing -> (if escape then acc else B.snoc acc '\\', B.empty)
         | h == '/' -> (acc, rest)
         | otherwise -> nextUntilSlash rest (B.snoc acc h) escape
-buildTermFromFunction "random-string" [arg] subst prog = do
+
+_randomString :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_randomString [arg] subst prog = do
   pat <- argToStrBytes arg subst prog
   str <- randomString pat
   pure (TeExpression (DataString (strToBts str)))
-buildTermFromFunction "random-string" _ _ _ = throwIO (userError "Function random-string() requires exactly 1 dataizable argument")
-buildTermFromFunction "size" [Y.ArgBinding (BiMeta meta)] subst _ = do
+_randomString _ _ _ = throwIO (userError "Function random-string() requires exactly 1 dataizable argument")
+
+_size :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_size [Y.ArgBinding (BiMeta meta)] subst _ = do
   bds <- buildBindingThrows (BiMeta meta) subst
   pure (TeExpression (DataNumber (numToBts (fromIntegral (length bds)))))
-buildTermFromFunction "size" _ _ _ = throwIO (userError "Function size() requires exactly 1 meta binding")
-buildTermFromFunction "tau" [Y.ArgExpression expr] subst prog = do
-  TeBytes bts <- buildTermFromFunction "dataize" [Y.ArgExpression expr] subst prog
+_size _ _ _ = throwIO (userError "Function size() requires exactly 1 meta binding")
+
+_tau :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_tau [Y.ArgExpression expr] subst prog = do
+  TeBytes bts <- _dataize [Y.ArgExpression expr] subst prog
   attr <- parseAttributeThrows (btsToUnescapedStr bts)
   pure (TeAttribute attr)
-buildTermFromFunction "tau" _ _ _ = throwIO (userError "Function tau() requires exactly 1 argument as expression")
-buildTermFromFunction "string" [Y.ArgExpression expr] subst _ = do
+_tau _ _ _ = throwIO (userError "Function tau() requires exactly 1 argument as expression")
+
+_string :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_string [Y.ArgExpression expr] subst _ = do
   (expr', _) <- buildExpressionThrows expr subst
   str <- case expr' of
     DataNumber bts -> pure (DataString (strToBts (either show show (btsToNum bts))))
@@ -159,16 +191,20 @@ buildTermFromFunction "string" [Y.ArgExpression expr] subst _ = do
             )
         )
   pure (TeExpression str)
-buildTermFromFunction "string" [Y.ArgAttribute attr] subst _ = do
+_string [Y.ArgAttribute attr] subst _ = do
   attr' <- buildAttributeThrows attr subst
   pure (TeExpression (DataString (strToBts (prettyAttribute attr'))))
-buildTermFromFunction "string" _ _ _ = throwIO (userError "Function string() requires exactly 1 argument as attribute or data expression (Φ̇.number or Φ̇.string)")
-buildTermFromFunction "number" [Y.ArgExpression expr] subst _ = do
+_string _ _ _ = throwIO (userError "Function string() requires exactly 1 argument as attribute or data expression (Φ̇.number or Φ̇.string)")
+
+_number :: [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_number [Y.ArgExpression expr] subst _ = do
   (expr', _) <- buildExpressionThrows expr subst
   case expr' of
     DataString bts -> do
       num <- parseNumberThrows (btsToUnescapedStr bts)
       pure (TeExpression num)
     _ -> throwIO (userError (printf "Function number() expects expression to be 'Φ̇.string', but got:\n%s" (prettyExpression' expr')))
-buildTermFromFunction "number" _ _ _ = throwIO (userError "Function number() requires exactly 1 argument as 'Φ̇.string'")
-buildTermFromFunction func _ _ _ = throwIO (userError (printf "Function %s() is not supported or does not exist" func))
+_number _ _ _ = throwIO (userError "Function number() requires exactly 1 argument as 'Φ̇.string'")
+
+_unsupported :: String -> [Y.ExtraArgument] -> Subst -> Program -> IO Term
+_unsupported func _ _ _ = throwIO (userError (printf "Function %s() is not supported or does not exist" func))
\ No newline at end of file
