@@ -4,7 +4,7 @@
 -- SPDX-FileCopyrightText: Copyright (c) 2025 Objectionary.com
 -- SPDX-License-Identifier: MIT
 
-module Rule where
+module Rule (RuleContext (..), isNF, matchProgramWithRule, meetCondition) where
 
 import Ast
 import Builder (buildAttribute, buildBinding, buildBindingThrows, buildExpression, buildExpressionThrows)
@@ -31,30 +31,6 @@ data RuleContext = RuleContext
     _buildTerm :: BuildTermFunc
   }
 
--- Check if given attribute is present in given binding
-attrInBindings :: Attribute -> [Binding] -> Bool
-attrInBindings attr (bd : bds) = attrInBinding attr bd || attrInBindings attr bds
-  where
-    attrInBinding :: Attribute -> Binding -> Bool
-    attrInBinding attr (BiTau battr _) = attr == battr
-    attrInBinding attr (BiVoid battr) = attr == battr
-    attrInBinding AtLambda (BiLambda _) = True
-    attrInBinding AtDelta (BiDelta _) = True
-    attrInBinding _ _ = False
-attrInBindings _ _ = False
-
--- Convert Number to Integer
-numToInt :: Y.Number -> Subst -> Maybe Integer
-numToInt (Y.Ordinal (AtMeta meta)) (Subst mp) = case M.lookup meta mp of
-  Just (MvAttribute (AtAlpha idx)) -> Just idx
-  _ -> Nothing
-numToInt (Y.Ordinal (AtAlpha idx)) subst = Just idx
-numToInt (Y.Length (BiMeta meta)) (Subst mp) = case M.lookup meta mp of
-  Just (MvBindings bds) -> Just (toInteger (length bds))
-  _ -> Nothing
-numToInt (Y.Literal num) subst = Just num
-numToInt _ _ = Nothing
-
 -- Returns True if given expression matches with any of given normalization rules
 -- Here we use unsafePerformIO because we're sure that conditions which are used
 -- in normalization rules doesn't throw an exception.
@@ -91,35 +67,69 @@ isNF (ExFormation bds) ctx = normalBindings bds || not (matchesAnyNormalizationR
             _ -> False
 isNF expr ctx = not (matchesAnyNormalizationRule expr ctx)
 
-meetCondition' :: Y.Condition -> Subst -> RuleContext -> IO [Subst]
-meetCondition' (Y.Or []) _ _ = pure []
-meetCondition' (Y.Or (cond : rest)) subst ctx = do
+_or :: [Y.Condition] -> Subst -> RuleContext -> IO [Subst]
+_or [] _ _ = pure []
+_or (cond : rest) subst ctx = do
   met <- meetCondition' cond subst ctx
   if null met
-    then meetCondition' (Y.Or rest) subst ctx
+    then _or rest subst ctx
     else pure met
-meetCondition' (Y.And []) subst _ = pure [subst]
-meetCondition' (Y.And (cond : rest)) subst ctx = do
+
+_and :: [Y.Condition] -> Subst -> RuleContext -> IO [Subst]
+_and [] subst _ = pure [subst]
+_and (cond : rest) subst ctx = do
   met <- meetCondition' cond subst ctx
   if null met
     then pure []
-    else meetCondition' (Y.And rest) subst ctx
-meetCondition' (Y.Not cond) subst ctx = do
+    else _and rest subst ctx
+
+_not :: Y.Condition -> Subst -> RuleContext -> IO [Subst]
+_not cond subst ctx = do
   met <- meetCondition' cond subst ctx
   pure [subst | null met]
-meetCondition' (Y.In attr binding) subst _ =
+
+_in :: Attribute -> Binding -> Subst -> RuleContext -> IO [Subst]
+_in attr binding subst _ =
   case (buildAttribute attr subst, buildBinding binding subst) of
-    (Just attr, Just bds) -> pure [subst | attrInBindings attr bds] -- if attrInBindings attr bd then [subst] else []
+    (Right attr, Right bds) -> pure [subst | attrInBindings attr bds] -- if attrInBindings attr bd then [subst] else []
     (_, _) -> pure []
-meetCondition' (Y.Alpha (AtAlpha _)) subst _ = pure [subst]
-meetCondition' (Y.Alpha (AtMeta name)) (Subst mp) _ = case M.lookup name mp of
+  where
+    -- Check if given attribute is present in given binding
+    attrInBindings :: Attribute -> [Binding] -> Bool
+    attrInBindings attr (bd : bds) = attrInBinding attr bd || attrInBindings attr bds
+      where
+        attrInBinding :: Attribute -> Binding -> Bool
+        attrInBinding attr (BiTau battr _) = attr == battr
+        attrInBinding attr (BiVoid battr) = attr == battr
+        attrInBinding AtLambda (BiLambda _) = True
+        attrInBinding AtDelta (BiDelta _) = True
+        attrInBinding _ _ = False
+    attrInBindings _ _ = False
+
+_alpha :: Attribute -> Subst -> RuleContext -> IO [Subst]
+_alpha (AtAlpha _) subst _ = pure [subst]
+_alpha (AtMeta name) (Subst mp) _ = case M.lookup name mp of
   Just (MvAttribute (AtAlpha _)) -> pure [Subst mp]
   _ -> pure []
-meetCondition' (Y.Alpha _) _ _ = pure []
-meetCondition' (Y.Eq (Y.CmpNum left) (Y.CmpNum right)) subst _ = case (numToInt left subst, numToInt right subst) of
+_alpha _ _ _ = pure []
+
+_eq :: Y.Comparable -> Y.Comparable -> Subst -> RuleContext -> IO [Subst]
+_eq (Y.CmpNum left) (Y.CmpNum right) subst _ = case (numToInt left subst, numToInt right subst) of
   (Just left_, Just right_) -> pure [subst | left_ == right_]
   (_, _) -> pure []
-meetCondition' (Y.Eq (Y.CmpAttr left) (Y.CmpAttr right)) subst _ = pure [subst | compareAttrs left right subst]
+  where
+      -- Convert Number to Integer
+    numToInt :: Y.Number -> Subst -> Maybe Integer
+    numToInt (Y.Ordinal (AtMeta meta)) (Subst mp) = case M.lookup meta mp of
+      Just (MvAttribute (AtAlpha idx)) -> Just idx
+      _ -> Nothing
+    numToInt (Y.Ordinal (AtAlpha idx)) subst = Just idx
+    numToInt (Y.Length (BiMeta meta)) (Subst mp) = case M.lookup meta mp of
+      Just (MvBindings bds) -> Just (toInteger (length bds))
+      _ -> Nothing
+    numToInt (Y.Literal num) subst = Just num
+    numToInt _ _ = Nothing
+_eq (Y.CmpAttr left) (Y.CmpAttr right) subst _ = pure [subst | compareAttrs left right subst]
   where
     compareAttrs :: Attribute -> Attribute -> Subst -> Bool
     compareAttrs (AtMeta left) (AtMeta right) (Subst mp) = case (M.lookup left mp, M.lookup right mp) of
@@ -132,7 +142,7 @@ meetCondition' (Y.Eq (Y.CmpAttr left) (Y.CmpAttr right)) subst _ = pure [subst |
       Just (MvAttribute found) -> attr == found
       _ -> False
     compareAttrs left right _ = right == left
-meetCondition' (Y.Eq (Y.CmpExpr left) (Y.CmpExpr right)) subst _ = pure [subst | compareExprs left right subst]
+_eq (Y.CmpExpr left) (Y.CmpExpr right) subst _ = pure [subst | compareExprs left right subst]
   where
     compareExprs :: Expression -> Expression -> Subst -> Bool
     compareExprs (ExMeta left) (ExMeta right) (Subst mp) = case (M.lookup left mp, M.lookup right mp) of
@@ -145,30 +155,38 @@ meetCondition' (Y.Eq (Y.CmpExpr left) (Y.CmpExpr right)) subst _ = pure [subst |
       Just (MvExpression found _) -> expr == found
       _ -> False
     compareExprs left right _ = left == right
-meetCondition' (Y.Eq _ _) _ _ = pure []
-meetCondition' (Y.NF (ExMeta meta)) (Subst mp) ctx = case M.lookup meta mp of
-  Just (MvExpression expr _) -> pure [Subst mp | isNF expr ctx]
+_eq _ _ _ _ = pure []
+
+_nf :: Expression -> Subst -> RuleContext -> IO [Subst]
+_nf (ExMeta meta) (Subst mp) ctx = case M.lookup meta mp of
+  Just (MvExpression expr _) -> _nf expr (Subst mp) ctx
   _ -> pure []
-meetCondition' (Y.NF expr) (Subst mp) ctx = pure [Subst mp | isNF expr ctx]
-meetCondition' (Y.XI (ExMeta meta)) (Subst mp) ctx = case M.lookup meta mp of
-  Just (MvExpression expr _) -> meetCondition' (Y.XI expr) (Subst mp) ctx
+_nf expr subst ctx = pure [subst | isNF expr ctx]
+
+_xi :: Expression -> Subst -> RuleContext -> IO [Subst]
+_xi (ExMeta meta) (Subst mp) ctx = case M.lookup meta mp of
+  Just (MvExpression expr _) -> _xi expr (Subst mp) ctx
   _ -> pure []
-meetCondition' (Y.XI (ExFormation _)) subst _ = pure [subst]
-meetCondition' (Y.XI ExThis) subst _ = pure []
-meetCondition' (Y.XI ExGlobal) subst _ = pure [subst]
-meetCondition' (Y.XI (ExApplication expr (BiTau attr texpr))) subst ctx = do
-  onExpr <- meetCondition' (Y.XI expr) subst ctx
-  onTau <- meetCondition' (Y.XI texpr) subst ctx
+_xi (ExFormation _) subst _ = pure [subst]
+_xi ExThis subst _ = pure []
+_xi ExGlobal subst _ = pure [subst]
+_xi (ExApplication expr (BiTau attr texpr)) subst ctx = do
+  onExpr <- _xi expr subst ctx
+  onTau <- _xi texpr subst ctx
   pure [subst | not (null onExpr) && not (null onTau)]
-meetCondition' (Y.XI (ExDispatch expr _)) subst ctx = meetCondition' (Y.XI expr) subst ctx
-meetCondition' (Y.Matches pat (ExMeta meta)) (Subst mp) ctx = case M.lookup meta mp of
-  Just (MvExpression expr _) -> meetCondition' (Y.Matches pat expr) (Subst mp) ctx
+_xi (ExDispatch expr _) subst ctx = _xi expr subst ctx
+
+_matches :: String -> Expression -> Subst -> RuleContext -> IO [Subst]
+_matches pat (ExMeta meta) (Subst mp) ctx = case M.lookup meta mp of
+  Just (MvExpression expr _) -> _matches pat expr (Subst mp) ctx
   _ -> pure []
-meetCondition' (Y.Matches pat expr) subst ctx = do
+_matches pat expr subst ctx = do
   (TeBytes tgt) <- _buildTerm ctx "dataize" [Y.ArgExpression expr] subst (Program expr)
   matched <- match (B.pack pat) (B.pack (btsToUnescapedStr tgt))
   pure [subst | matched]
-meetCondition' (Y.PartOf exp bd) subst _ = do
+
+_partOf :: Expression -> Binding -> Subst -> RuleContext -> IO [Subst]
+_partOf exp bd subst _ = do
   (exp', _) <- buildExpressionThrows exp subst
   bds <- buildBindingThrows bd subst
   pure [subst | partOf exp' bds]
@@ -179,6 +197,18 @@ meetCondition' (Y.PartOf exp bd) subst _ = do
     partOf expr (BiTau _ expr' : rest) = expr == expr' || partOf expr rest
     partOf expr (bd : rest) = partOf expr rest
 
+meetCondition' :: Y.Condition -> Subst -> RuleContext -> IO [Subst]
+meetCondition' (Y.Or conds) = _or conds
+meetCondition' (Y.And conds) = _and conds
+meetCondition' (Y.Not cond) = _not cond
+meetCondition' (Y.In attr binding) = _in attr binding
+meetCondition' (Y.Alpha attr) = _alpha attr
+meetCondition' (Y.Eq left right) = _eq left right
+meetCondition' (Y.NF expr) = _nf expr
+meetCondition' (Y.XI expr) = _xi expr
+meetCondition' (Y.Matches pat expr) = _matches pat expr
+meetCondition' (Y.PartOf expr bd) = _partOf expr bd
+
 -- For each substitution check if it meetCondition to given condition
 -- If substitution does not meet the condition - it's thrown out
 -- and is not used in replacement
